
/*
 * Copyright (c) 2005 by Pascal de Bruijn 
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General 
 * Public License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

using System;
using System.Runtime.InteropServices;

namespace ClamAv
{
  public struct ClamAvLimits 
  {
    public uint   MaximumRecursionLevel;
    public uint   MaximumFiles;
    public uint   MaximumRatio; 
    public ushort ArchiveMemoryLimit;
    public ulong  MaximumFileSize; 
  }

  public enum ScanOptions : uint
  {
    ScanRaw            =   0,
    ScanArchive        =   1,
    ScanMail           =   2,
    ScanDisableRar     =   4,
    ScanOle2           =   8,
    ScanBlockEncrypted =  16,
    ScanHtml           =  32,
    ScanPe             =  64,
    ScanBlockBroken    = 128,
    ScanMailUrl        = 256,
    ScanBlockMax       = 512
  }
  
  unsafe public class ClamAv : IDisposable
  {  
    const int CL_CLEAN     =   0; /* virus not found */
    const int CL_VIRUS     =   1; /* virus found */
  
    const int CL_EMAXREC   =  10; /* recursion level limit exceeded */
    const int CL_EMAXSIZE  =  11; /* size limit exceeded */
    const int CL_EMAXFILES =  12; /* files limit exceeded */
    const int CL_ERAR      = 100; /* rar handler error */
    const int CL_EZIP      = 101; /* zip handler error */
    const int CL_EMALFZIP  = 102; /* malformed zip */
    const int CL_EGZIP     = 103; /* gzip handler error */
    const int CL_EBZIP     = 104; /* bzip2 handler error */
    const int CL_EOLE2     = 105; /* OLE2 handler error */
    const int CL_EMSCOMP   = 106; /* compress.exe handler error */
    const int CL_EMSCAB    = 107; /* MS CAB module error */
    const int CL_EACCES    = 200; /* access denied */
    const int CL_ENULLARG  = 300; /* null argument error */
  
    const int CL_ETMPFILE  =  -1; /* tmpfile() failed */
    const int CL_EFSYNC    =  -2; /* fsync() failed */
    const int CL_EMEM      =  -3; /* memory allocation error */
    const int CL_EOPEN     =  -4; /* file open error */
    const int CL_EMALFDB   =  -5; /* malformed database */
    const int CL_EPATSHORT =  -6; /* pattern too short */
    const int CL_ETMPDIR   =  -7; /* mkdir() failed */
    const int CL_ECVD      =  -8; /* not a CVD file (or broken) */
    const int CL_ECVDEXTR  =  -9; /* CVD extraction failure */
    const int CL_EMD5      = -10; /* MD5 verification error */
    const int CL_EDSIG     = -11; /* digital signature verification error */
    const int CL_EIO       = -12; /* general I/O error */
    const int CL_EFORMAT   = -13; /* bad format or broken file */
    
    private IntPtr Root;
    
    public uint SignatureCount;
    public ClamAvLimits Limits;
    public ScanOptions Options;

    const string Library = "clamav";
  
    [DllImport(Library)]
    private extern static int cl_scanfile (string filename, ref IntPtr virname, ref ulong scanned, IntPtr root, ref ClamAvLimits limits, uint options);
    
    [DllImport(Library)]
    private extern static int cl_retflevel (); 
    
    [DllImport(Library)]
    private extern static IntPtr cl_retver ();
    
    [DllImport(Library)]
    private extern static int cl_loaddb (string filename, ref IntPtr root, ref uint signo);
    
    [DllImport(Library)]
    private extern static int cl_loaddbdir (string dirname, ref IntPtr root, ref uint signo);
    
    [DllImport(Library)]
    private extern static IntPtr cl_retdbdir ();
    
    [DllImport(Library)]
    private extern static void cl_debug ();
    
    [DllImport(Library)]
    private extern static void cl_settempdir (string dir, short leavetemps);
    
    [DllImport(Library)]
    private extern static int cl_build (IntPtr root);  
    
    [DllImport(Library)]
    private extern static void cl_free (IntPtr root);
    
    [DllImport(Library)]
    private extern static IntPtr cl_strerror (int clerror);
  
    public ClamAv (string DatabaseDirectory)
    {
	  Root = (IntPtr)null;
	  SignatureCount = 0;
		
      ThrowException (cl_loaddbdir (DatabaseDirectory, ref Root, ref SignatureCount));    
      ThrowException (cl_build (Root));
    
      Limits.MaximumRecursionLevel = 5;
      Limits.MaximumFiles = 1000;
      Limits.MaximumRatio = 200;
      Limits.ArchiveMemoryLimit = 0;
      Limits.MaximumFileSize = 10 * 1024 * 1024;
    
      Options = (ScanOptions.ScanArchive | ScanOptions.ScanMail | ScanOptions.ScanOle2 | ScanOptions.ScanHtml | ScanOptions.ScanPe);
    }
    
    public ClamAv () : this (Marshal.PtrToStringAnsi (cl_retdbdir ())) {}
  
    public string DatabaseDirectory ()
    {
      return Marshal.PtrToStringAnsi (cl_retdbdir ());
    }
  
    public bool ScanFile (string Path, out string VirusName)
    {
      int ReturnCode;
      
      IntPtr VirusNamePtr = (IntPtr)null;
      ulong Scanned = 0;

      ReturnCode = ThrowException(cl_scanfile (Path, ref VirusNamePtr, ref Scanned, Root, ref Limits, (uint)Options));
      
      VirusName = Marshal.PtrToStringAnsi (VirusNamePtr);
    
      switch (ReturnCode)
      {
        case CL_VIRUS:
          return true;
        default:
          return false;
      }
    }
  
    private int ThrowException (int ResultCode)
    {   
      switch (ResultCode)
      {
        case CL_CLEAN:                                        /* virus not found */
          return ResultCode;
        case CL_VIRUS:                                        /* virus found */
          return ResultCode;
        case CL_EMAXREC:                                      /* recursion level limit exceeded */
          throw new RecursionLevelLimitException ();
        case CL_EMAXSIZE:                                     /* size limit exceeded */
          throw new SizeLimitException ();
        case CL_EMAXFILES:                                    /* files limit exceeded */
          throw new FilesLimitException ();
        case CL_ERAR:                                         /* rar handler error */
          throw new RarHandlerException ();
        case CL_EZIP:                                         /* zip handler error */
          throw new ZipHandlerException ();
        case CL_EMALFZIP:                                     /* malformed zip */
          throw new MalformedZipException ();
        case CL_EGZIP:                                        /* gzip handler error */
          throw new GzipHandlerException ();
        case CL_EBZIP:                                        /* bzip2 handler error */
          throw new Bzip2HandlerException ();
        case CL_EOLE2:                                        /* OLE2 handler error */
          throw new Ole2HandlerException ();
        case CL_EMSCOMP:                                      /* compress.exe handler error */
          throw new CompressHandlerException ();
        case CL_EMSCAB:                                       /* MS CAB module error */
          throw new CabHandlerException ();
        case CL_EACCES:                                       /* access denied */
          throw new AccessDeniedException ();
        case CL_ENULLARG:                                     /* null argument error */
          throw new NullArgumentException ();
        case CL_ETMPFILE:                                     /* tmpfile() failed */
          throw new TmpfileException ();
        case CL_EFSYNC:                                       /* fsync() failed */
          throw new FsyncException ();
        case CL_EMEM:                                         /* memory allocation error */
          throw new MemoryAllocationException ();
        case CL_EOPEN:                                        /* file open error */
          throw new FileOpenException ();
        case CL_EMALFDB:                                      /* malformed database */
          throw new MalformedDatabaseException ();
        case CL_EPATSHORT:                                    /* pattern too short */
          throw new PatternTooShortException ();
        case CL_ETMPDIR:                                      /* mkdir() failed */
          throw new MkdirException ();
        case CL_ECVD:	                                      /* not a CVD file (or broken) */
          throw new BrokenCvdException ();
        case CL_ECVDEXTR:	                                  /* CVD extraction failure */
          throw new CvdExtractionException ();
        case CL_EMD5:	                                      /* MD5 verification error */
          throw new Md5VerificationException ();
        case CL_EDSIG:                                        /* digital signature verification error */
          throw new DigitalSignatureVerificationException ();
        case CL_EIO:                                          /* general I/O error */
          throw new GeneralIoException ();
        case CL_EFORMAT:                                      /* bad format or broken file */
          throw new BadFormatException ();
        default:                                              /* unknown error */
          throw new UnknownException ();
      }  
    }
  
    private void Free ()
    {
      cl_free (Root);  
    } 

    public void Dispose ()
    { 
      Free ();
      System.GC.SuppressFinalize (this);    
    }
  
    ~ClamAv ()
    {
      Free ();
    }    
  }
}